use backtrack::{BackTracker, Reine};

#[derive(Clone, Copy)]
pub enum HeuristicsApproach {
    Opportunist,
    Failing,
}

pub fn nb_blocked_lines(n: usize, to_put: Reine, put: &BackTracker) -> usize {
    let row = to_put.0 + 1;

    if row as usize >= n {
        return 0;
    }

    let mut blocked: Vec<i32> = Vec::new();

    blocked.append(&mut blocked_columns(row, &to_put));

    for reine in put {
        let cols = blocked_columns(row, reine);

        //only add columns that aren't
        //already in blocked
        for col in cols {
            if !blocked.contains(&col) {
                blocked.push(col);
            }
        }
    }

    return blocked
        .iter()
        .filter(|c| **c >= 0 && **c < n as i32)
        .count();
}

fn blocked_columns(row: i32, reine: &Reine) -> Vec<i32> {
    let row_diff = (row - reine.0).abs();
    vec![reine.1 - row_diff, reine.1, reine.1 + row_diff]
}

mod tests {
    #[test]
    fn test_first_queen_2x2_checkboard() {
        let backtracker: BackTracker = vec![];
        let to_put: Reine = (0, 0);
        let board_size: usize = 2;

        assert_eq!(nb_blocked_lines(board_size, to_put, &backtracker), 2);
    }

    #[test]
    fn test_second_queen_3x3_checkboard() {
        let backtracker: BackTracker = vec![(0, 0)];
        let to_put: Reine = (1, 2);
        let board_size: usize = 3;

        assert_eq!(nb_blocked_lines(board_size, to_put, &backtracker), 3);
    }

    #[test]
    fn test_fourth_queen_6x6_checkboard() {
        let backtracker: BackTracker = vec![(0, 0), (1, 2), (2, 4)];
        let to_put: Reine = (3, 1);
        let board_size: usize = 6;

        assert_eq!(nb_blocked_lines(board_size, to_put, &backtracker), 5);
    }

    #[test]
    fn test_fourth_queen_4x4_checkboard() {
        let backtracker: BackTracker = vec![(0, 1), (1, 3), (2, 0)];
        let to_put: Reine = (3, 0);
        let board_size: usize = 4;

        assert_eq!(nb_blocked_lines(board_size, to_put, &backtracker), 0);
    }
}
